/*
 * LCD Frequency counter and amplitude meter
 * For LCD gate-dip oscillator
 * uses parallel 8 char x 2 line LCD display
 * Timebase is generated by wrapping serial Rx to Tx
 * and generating serial data to make serial Rx interrupt.
 *
 * Line two of display starts at char position 0x4
 * 
 * Timer T1 is used to count clocks from external input
 * External freq input is divided by 32 to clock T1
 * This allows frequencies up to 100 MHz
 *
 * To avoid time-consuming muliplication, the timebase
 * is set at 312.5 Hz.  This makes a 1 MHz input (after /32 division)
 * result in 100 counts in the T1 counter. 100 MHz is 10,000 counts.
 *
 * Serial Tx/Rx pin is used to generate 1565 Hz tone for modulaton.
 * So value of 0x55 is sent to serial Tx to make the tone.
 *
 * Uses internal Mega8535 ADC for measuring amplitude.
 */

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "iocompat.h"	


volatile uint8_t Flags = 0	;
	// Bit 0: 1 = freq, 0 = period
	// Bit 1: 1 = 50% mode

volatile uint8_t T0Ticker  = 0 ;			// Sync byte
volatile uint8_t LCDTimer = 0 ;
volatile uint8_t LCDSize = 0 ;		// Bytes in buffer
volatile uint8_t LCDIndex = 0 ;		// Index into buffer
volatile uint8_t LockoutCount = 0 ;	// Ticks to lock out LCD after command.
volatile uint8_t LCDStatus = 0 ;		// For reading busy and addr bits
volatile uint16_t TestCounter = 0 ;
volatile uint8_t RxByte ;				// USART receive byte
volatile uint8_t SampleTime ;			// Sample interrupt flag.
volatile uint8_t SampleCount ;			// Count of samples for display rate.
volatile int16_t Count ;				// Timer 1 capture value
volatile int16_t DisplayCount ;			// Averaged count
volatile int16_t T1OldValue ;			// Previous T1 value
volatile int16_t T1Count ;				// Current T1 count.
volatile int16_t Average ;				// Intermediate value for averaging counts.
volatile uint8_t AddrFlag = 0 ;			// Indicates when addr switched.
volatile uint8_t ADCValue = 0 ;			// 8-bit LCD value for bar-graph display
volatile uint16_t BarGraphTemp ;		// Intermediate math value for bargraph calc.
volatile uint8_t ADCReady = 0;			// 1 to indicate ADC ready.
volatile uint8_t BarGraphByte ;

volatile uint8_t LCDBuffer[16] ;
volatile uint8_t DecBuffer[5] ;			// For hex - to decimal-ascii conversion

// Define number of T0 rollovers for 1 Hz,
// based on 15 MHz clock and /1024 prescaler
#define HzCount (15000000/256/1024)

// LCD control pins
#define LCDCE 0x80
#define LCDRW 0x40
#define LCDRS 0x20
#define LCDCtlPort PORTD
#define LCDDataPort PORTC
#define LCDPin PINC
#define LCDDir DDRC


// Number of characters in bargraph display
// Multiplied by 5 to get number of segments.
#define BarGraphChars 8 




// Initialization
void ioinit (void)			
{

	// Port A, ADC
	DDRA = 0x00 ;
	PORTA = 0x00 ;			

	// PortB is ISP port
	// Bit 6 is also heartbeat LED
	// PortB.4 is sensitivity; 0 = high sens.
	DDRB = 0x40 ;
	PORTB = 0 ;

	// PORTC is LCD data port
	DDRC = 0x0 ;		// Initially all inputs
	PORTC = 0 ;

	// PortD is control lines to LCD
	// Bit 7 = -CE
	// Bit 6 = R/W
	// Bit 5 = Data/Code
	DDRD = 0xFE ;
	PORTD = 0x00 ;



	// Set up timer T0 as free-running counter,
	// using /1024 divider
	// 57 Hz with 15 MHz crystal
	// Other dividers:
	// 1 = No divisor  (T0 clock = crystal freq)
	// 2 = /8
	// 3 = /64
	// 4 = /256
	// 5 = /1024
	TCCR0 = 5 ;



	// CTC  mode
	TCCR1A = 0x00  ;
	TCCR1B = 0x07 ;		// Ext clk
	OCR1A = 0x0 ;
	OCR1B = 0x0 ;


	// Set up UART for 3125 baud operation, no parity, 8 data bits.
	// UART is just for producing sampling interrupt

	UCSRB = 0x98 ;				// Enable transmitter, receiver, Rx interupt
	UBRRH = 299 >> 8 ;
	UBRRL = 299 & 0xFF ;
	UCSRC = 0x86 ;				// 8 data bits, 1 stop bit, no parity

	// Set up ADC for 2.5V internal reference, channel 0
	ADMUX = 0xC0 ;
	ADCSRA = 0x83 ;


}

void LCDWait()
{
	if ((TIFR & 0x2) != 0)
	{
		TIFR = 2 ;
		LCDTimer++ ;
	}
}


void LCDDelay(void)
{
	asm("nop \n") ;
	asm("nop \n") ;
	asm("nop \n") ;
	asm("nop \n") ;
}


// Send command byte to lCD
void SendLCDCmd(uint8_t CommandByte) 
{
	LCDCtlPort &= ~(LCDRW | LCDRS) ;	// RS and RW low
	asm("nop \n") ;
	LCDCtlPort &= ~LCDCE ;		// Enab low
	LCDDir = 0xFF ;
	LCDDataPort = CommandByte ;
	asm ("nop \n") ;
	asm ("nop \n") ;
	LCDCtlPort |= LCDCE ;		// Enab bit hi
	LCDDelay() ;
	LCDCtlPort &= ~LCDCE ;		// Enab bit low
	asm ("nop \n") ;
	LCDDir = 0x00 ;				// PortA input

}

// Send data byte to LCD
void SendLCDData(uint8_t DataByte) 
{
	LCDCtlPort &= ~LCDRW ;		// Set read/write control bit low
	asm("nop \n") ;
	LCDCtlPort &= ~LCDCE ;		// Enab low
	LCDDir = 0xFF ;
	LCDCtlPort |= LCDRS ;		// RS bit to 1 (data)
	LCDDataPort = DataByte ;
	asm ("nop \n") ;
	asm ("nop \n") ;
	LCDCtlPort |= LCDCE ;		// Enab bit hi
	LCDDelay() ;
	LCDCtlPort &= ~LCDCE ;		// Enab bit low
	asm ("nop \n") ;
	LCDDir = 0x00 ;				// PortA input
}

// Return state of LCD busy flag and addr counter
uint8_t GetLCDStatus(void)
{
	uint8_t result;
	
	LCDDir = 0 ;		// Input mode

	// Set up LCD to read status bit on D7
	LCDCtlPort |= LCDRW ;		// RW 1	
	asm("nop \n") ;
	LCDCtlPort &= ~LCDRS ;		// RS 0
	asm("nop \n") ;
	LCDCtlPort |= LCDCE ;		// Enab 1 to read port
	LCDDelay() ;
	result = LCDPin ;
	LCDCtlPort &= ~LCDCE ;
	
	return (result) ;

}

// Wait for char ready based on "ready" bit.
void LCDCharWait()
{
	uint8_t LStatus ;

	LStatus = 0x80 ;
	while ((LStatus & 0x80) != 0) LStatus = GetLCDStatus() ;

}


void SetCGRamAddr(uint8_t Addr)
{
	SendLCDCmd(0x40 + Addr) ;
}

// Send LCD cursor to home position
void LCDHome(void) 
{
	SendLCDCmd(0x2) ;
}

// Clear display and set addr to 0
void LCDClear(void)
{
	SendLCDCmd(0x1) ;
}

	// Set up special characters
	// Special characters turn each char into 0-5 amplitude marks.
	// char:        1   2   3     4      5
	// Display:     |  ||  |||  ||||  |||||
	// Note that all segs are half a char high except 5, which is
	// full char high.
	// So two row values are passed in, the value for rows 3-6,
	// and the value for rows 1-2.
void CreateAChar(uint8_t RowValue, uint8_t StartAddr, uint8_t RowValue2)
{
	// First row, blank
	SetCGRamAddr(StartAddr) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;
	SendLCDData(0) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;

	// Second row 
	SetCGRamAddr(StartAddr + 1) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;
	SendLCDData(RowValue2) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;

	// Third row blank
	SetCGRamAddr(StartAddr + 2) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;
	SendLCDData(RowValue2) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;

	// Fourth row 
	SetCGRamAddr(StartAddr + 3) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;
	SendLCDData(RowValue) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;


	// Fifth row 
	SetCGRamAddr(StartAddr + 4) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;
	SendLCDData(RowValue) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;


	// Sixth
	SetCGRamAddr(StartAddr + 5) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;
	SendLCDData(RowValue) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;


	// Seventh row. 
	SetCGRamAddr(StartAddr + 6) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;
	SendLCDData(RowValue) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;


	// Seventh row.
	SetCGRamAddr(StartAddr + 7) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;
	SendLCDData(0) ;
	LCDTimer = 0 ;
	LCDCharWait() ;
	//while (LCDTimer < 2) LCDWait() ;

}

void LCDInit(void)
{
	// Initialize LCD
	LCDDir = 0xFF ;		// All outputs

	// Kill 30 ms
	LCDTimer = 0 ;
	while (LCDTimer < 2) LCDWait() ;

	SendLCDCmd(0x3E) ;
	// Kill 30 ms
	LCDTimer = 0 ;
	while (LCDTimer < 2) LCDWait() ;

	SendLCDCmd(0x3E) ;
	// Kill 30 ms
	LCDTimer = 0 ;
	while (LCDTimer < 2) LCDWait() ;

	LCDTimer = 0 ;
	while (LCDTimer < 2) LCDWait() ;


	SendLCDCmd(0x1) ;
	// Kill 30 ms
	LCDTimer = 0 ;
	while (LCDTimer < 2) LCDWait() ;


	SendLCDCmd(0x2) ;
	// Kill 30 ms
	LCDTimer = 0 ;
	while (LCDTimer < 2) LCDWait() ;


	SendLCDCmd(0x0E) ;
	// Kill 30 ms
	LCDTimer = 0 ;
	while (LCDTimer < 2) LCDWait() ;



	// Now write char "A" to display
	LCDCtlPort |= LCDRS ;
	asm("nop \n") ;
	LCDCtlPort |= LCDCE ; 
	LCDDataPort = 0x41 ;
	LCDCtlPort &= ~(LCDRW) ;
	asm ("nop \n") ;
	LCDCtlPort &= ~LCDCE ;
	LCDDelay() ;

	// Kill 30 ms
	LCDTimer = 0 ;
	while (LCDTimer < 2) LCDWait() ;


	// Now write char "B" to display
	LCDCtlPort |= LCDRS ;
	asm("nop \n") ;
	LCDCtlPort |= LCDCE ; 
	LCDDataPort = 0x42 ;
	LCDCtlPort &= ~(LCDRW) ;
	asm ("nop \n") ;
	LCDCtlPort &= ~LCDCE ;
	LCDDelay() ;

	// Kill 30 ms
	LCDTimer = 0 ;
	while (LCDTimer < 2) LCDWait() ;

	// Now write char "C" to display
	LCDCtlPort |= LCDRS ;
	asm("nop \n") ;
	LCDCtlPort |= LCDCE ; 
	LCDDataPort = 0x43 ;
	LCDCtlPort &= ~(LCDRW) ;
	asm ("nop \n") ;
	LCDCtlPort &= ~LCDCE ;
	LCDDelay() ;


	// Kill 30 ms
	LCDTimer = 0 ;
	while (LCDTimer < 2) LCDWait() ;

	LCDDataPort = 0x0 ;

	LCDCtlPort |= LCDRW ;		// Make port read so status works.


	// Create special characters for bar-graph display
	// Parameters: Row value, rows 306, start CG Ram addr, Row value, rows 1-2
	CreateAChar(0, 0, 0) ;
	CreateAChar(0x10, 8, 0) ;
	CreateAChar(0x18, 16, 0) ;
	CreateAChar(0x1C, 24, 0) ;
	CreateAChar(0x1E, 32, 0) ;
	CreateAChar(0x1F, 40, 0x1) ;


	// Write chars 12345 to buffer so we can test LCD writing from main code.
	// If this doesn't work, the "ABC" will be on display from above code.
	// If it does work, then "12345" will be on display.
	LCDBuffer[0] = 0x31 ;
	LCDBuffer[1] = 0x32 ;
	LCDBuffer[2] = 0x33 ;
	LCDBuffer[3] = 0x34 ;
	LCDBuffer[4] = 0x35 ;
	LCDSize = 5 ;
	LCDIndex = 0 ;
	LCDClear() ;

}

// Convert one hex nybble to ASCII
// If char <= 9, add 30h, if > 9, add 55
uint8_t Hex2Ascii(uint8_t HexByte)
{
	uint8_t result ;
	if ((HexByte & 0x0F) > 0x9) result = (HexByte & 0x0F) + 55 ;
	else result = (HexByte & 0x0F) + 0x30 ;
	return(result) ;
}

// Convert hex value to decimal-ascii in DecBuffer
void Hex2DecAscii(uint16_t HexValue)
{
	uint8_t i ;			// Loop counter
	uint16_t TempValue  = HexValue;
	uint8_t CarryBit1 ;	// Alternating carry bits for value shift.
	uint8_t CarryBit2 ;
	DecBuffer[0] = 0 ;
	DecBuffer[1] = 0 ;
	DecBuffer[2] = 0 ;
	DecBuffer[3] = 0 ;
	DecBuffer[4] = 0 ;

	// Shift input value left one bit at a time into DecBuffer,
	// with DecBuffer[0] being LS byte
	// If any value in any DecBuffer element is >5, add 3 to it.
	for (i = 0; i <16; i++)
	{
		if ((TempValue & 0x8000) != 0) CarryBit1 = 1 ;
		else CarryBit1 = 0 ;
		TempValue = TempValue << 1 ;

		// Shift all five digits to the left
		if ((DecBuffer[0] & 0x8) > 0) CarryBit2 = 1 ;
		else CarryBit2 = 0 ;
		DecBuffer[0] = ((DecBuffer[0] << 1) + CarryBit1) & 0x0F ;

		if ((DecBuffer[1] & 0x8) > 0) CarryBit1 = 1 ;
		else CarryBit1 = 0 ;
		DecBuffer[1] = ((DecBuffer[1] << 1) + CarryBit2) & 0x0F ;	

		if ((DecBuffer[2] & 0x8) > 0) CarryBit2 = 1 ;
		else CarryBit2 = 0 ;
		DecBuffer[2] = ((DecBuffer[2] << 1) + CarryBit1) & 0x0F ;
	
		if ((DecBuffer[3] & 0x8) > 0) CarryBit1 = 1 ;
		else CarryBit1 = 0 ;
		DecBuffer[3] = ((DecBuffer[3] << 1) + CarryBit2) & 0x0F ;

		DecBuffer[4] = ((DecBuffer[4] << 1) + CarryBit1) & 0x0F ;


		// Now adjust for bcd; if any DecBuffer element >4, add 3.
		if (i < 15)
		{
			if (DecBuffer[0] > 4) DecBuffer[0] += 3 ;
			if (DecBuffer[1] > 4) DecBuffer[1] += 3 ;
			if (DecBuffer[2] > 4) DecBuffer[2] += 3 ;
			if (DecBuffer[3] > 4) DecBuffer[3] += 3 ;
			if (DecBuffer[4] > 4) DecBuffer[4] += 3 ;
		}

	}
	// Now convert to ASCII for display
	DecBuffer[0] = Hex2Ascii(DecBuffer[0]) ;	
	DecBuffer[1] = Hex2Ascii(DecBuffer[1]) ;	
	DecBuffer[2] = Hex2Ascii(DecBuffer[2]) ;	
	DecBuffer[3] = Hex2Ascii(DecBuffer[3]) ;	
	DecBuffer[4] = Hex2Ascii(DecBuffer[4]) ;	

	// make leading zeros into spaces, except for last two digits.
	if (DecBuffer[4] == 0x30)	
	{
		DecBuffer[4] = 0x20 ;
		if (DecBuffer[3] == 0x30)
		{
			DecBuffer[4] = 0x20 ;
			if (DecBuffer[3] == 0x30)
			{
				DecBuffer[3] = 0x20 ;
				if (DecBuffer[2] == 0x30) DecBuffer[2] = 0x20 ;
			}
		}
	}
}


int main (void)
{

    ioinit ();
	LCDInit() ;
	UDR = 0 ;
	sei() ;

	// 8-bit T0 ticker value, sets 1 Hz rollover
	static uint8_t T0Ticker;	
	T0Ticker = 0 ;

    /* loop forever, blinking the LEDs */

    for (;;)
	{	
	
		// If T0 rolls over, reset T0 overflow flag
		// and incr T0ticker.
		// If T0Ticker reaches half of 1 Hz count,
		// turn on LED.  If T0ticker reaches 1 Hz
		// count, turn off LED and reset ticker.
		if ((TIFR & 0x2) != 0)
		{
			TIFR = 2 ;
			T0Ticker++ ;

			// One conversion per T0 rollover.
			BarGraphTemp = ADC ;			// Read ADC
	
			// Divide by 2 or 4, depending on sensitivity setting.
			if ((PINB & 0x10) == 0x10)
				ADCValue = BarGraphTemp >> 2 ;	// Shift right to make byte.
			else ADCValue = BarGraphTemp >> 1 ;
			ADCSRA |= 0x40 ;				// Start next conversion
			ADCReady = 1 ;


			if (LockoutCount > 0) LockoutCount = LockoutCount - 1 ;
			if (T0Ticker == HzCount/2) PORTB |= 0x40 ;
			else
			{
				if (T0Ticker == HzCount)
				{
					PORTB &= 0xBF ;
					T0Ticker = 0 ;


					// Just for testing LCD, increment counter and display
				//	TestCounter++ ;
				//	if (TestCounter > 255) TestCounter = 0 ;
				//	LCDClear() ;
				//	LCDBuffer[0] = Hex2Ascii(TestCounter >> 4) ;
				//	LCDBuffer[1] = Hex2Ascii(TestCounter) ;
				//	LCDBuffer[2] = 0x20 ;
				//	LCDBuffer[3] = 0x20 ;
				//	LCDBuffer[4] = 0x20 ;
				//	LCDSize = 5 ;
				//	LCDIndex = 0 ;

				//	Hex2DecAscii(TestCounter) ;
				//	LCDBuffer[0] = DecBuffer[4];
				//	LCDBuffer[1] = DecBuffer[3];
				//	LCDBuffer[2] = DecBuffer[2];
				//	LCDBuffer[3] = DecBuffer[1];
				//	LCDBuffer[4] = DecBuffer[0];
				//	LCDSize = 5 ;
				//	LCDIndex = 0 ;

				}
			}

		}

		if (SampleTime == 1)
		{
			SampleTime = 0 ;
			SampleCount++ ;

			// Average samples
			DisplayCount = (Average + Count)/2 ;
			Average = Count ;

			// Update display with new value.
			if (SampleCount >= 40)
			{
				SampleCount = 0 ;
				LCDClear() ;
				Hex2DecAscii(DisplayCount) ;
				LCDBuffer[0] = DecBuffer[4];
				LCDBuffer[1] = DecBuffer[3];
				LCDBuffer[2] = DecBuffer[2];
				LCDBuffer[3] = '.' ;
				LCDBuffer[4] = DecBuffer[1];
				LCDBuffer[5] = DecBuffer[0];


				BarGraphTemp = ((BarGraphChars * 5) + 1) * ADCValue ;
				BarGraphByte = BarGraphTemp/256 ;

				if (BarGraphByte >= 5) 
				{
					LCDBuffer[8] = 5 ;
					BarGraphByte = BarGraphByte - 5 ;
				}
				else 
				{
					LCDBuffer[8] = BarGraphByte ;
					BarGraphByte = 0 ;
				}

				if (BarGraphByte >= 5) 
				{
					LCDBuffer[9] = 5 ;
					BarGraphByte = BarGraphByte - 5 ;
				}
				else 
				{
					LCDBuffer[9] = BarGraphByte ;
					BarGraphByte = 0 ;
				}				
				if (BarGraphByte >= 5) 
				{
					LCDBuffer[10]= 5 ;
					BarGraphByte = BarGraphByte - 5 ;
				}
				else 
				{
					LCDBuffer[10]= BarGraphByte ;
					BarGraphByte = 0 ;
				}				

				if (BarGraphByte >= 5) 
				{
					LCDBuffer[11]= 5 ;
					BarGraphByte = BarGraphByte - 5 ;
				}
				else 
				{
					LCDBuffer[11]= BarGraphByte ;
					BarGraphByte = 0 ;
				}

				if (BarGraphByte >= 5) 
				{
					LCDBuffer[12]= 5 ;
					BarGraphByte = BarGraphByte - 5 ;
				}
				else 
				{
					LCDBuffer[12]= BarGraphByte ;
					BarGraphByte = 0 ;
				}				
				
				if (BarGraphByte >= 5) 
				{
					LCDBuffer[13]= 5 ;
					BarGraphByte = BarGraphByte - 5 ;
				}
				else 
				{
					LCDBuffer[13]= BarGraphByte ;
					BarGraphByte = 0 ;
				}				

				
				if (BarGraphByte >= 5) 
				{
					LCDBuffer[14]= 5 ;
					BarGraphByte = BarGraphByte - 5 ;
				}
				else 
				{
					LCDBuffer[14]= BarGraphByte ;
					BarGraphByte = 0 ;
				}
				
				
				if (BarGraphByte >= 5) 
				{
					LCDBuffer[15]= 5 ;
					BarGraphByte = BarGraphByte - 5 ;
				}
				else 
				{
					LCDBuffer[15]= BarGraphByte ;
					BarGraphByte = 0 ;
				}


				// Temporarily send bar-graph codes for display
				//LCDBuffer[6] = ' ' ;
				//LCDBuffer[7] = ' ' ;
				//LCDBuffer[8] = 0 ;
				//LCDBuffer[9] = 1 ;
				//LCDBuffer[10] = 2 ;
				//LCDBuffer[11] = 3 ;
				//LCDBuffer[12] = 4 ;
				//LCDBuffer[13] = 5 ;


				LCDSize = 20 ;
				LCDIndex = 0 ;
				AddrFlag = 0 ;
			}


		}



		// Process LCD, if there is anything to send
		if (LCDSize > 0)
		{
			// Check for LCD busy.
			LCDStatus = GetLCDStatus() ;
			if ((LCDStatus & 0x80) == 0)
			{
				// If LCD index is 8, then need to bump DDRAM
				// address to 0x40.  LCD is 16 chars, 
				// but is configured as two rows of 8, elecrically.
				if ((LCDIndex == 8) && (AddrFlag == 0))
				{
					SendLCDCmd(0xC0) ;
					AddrFlag = 1 ;
				}
				else
				{
					SendLCDData(LCDBuffer[LCDIndex]) ;
					LCDIndex += 1 ;
					LCDSize = LCDSize - 1 ;
					if (LCDIndex > sizeof(LCDBuffer)) 
					{
						LCDIndex = 0 ;
					}
					if (LCDIndex == 0) AddrFlag = 0 ;
				}

			}
		}


	}
    return (0);
}


ISR(USART_RX_vect)
{

	RxByte = UDR ;
	UDR = 0x55 ;
	SampleTime = 1 ;

	T1OldValue = T1Count ;
	T1Count = TCNT1 ;
	Count = T1OldValue - T1Count ;
	if (Count < 0) Count = T1Count - T1OldValue ;

}
